#!/usr/bin/perl
#
#     Caption: Zaval File Search
#     $Revision: 1.17 $
#     $Date: 2002/02/16 12:30:17 $
#
#     @author:     Alex Buloichik, Victor Krapivin
#     @version:    1.2
#
# The Zaval File Search solution is a local area network tool designed 
# for fast file search on SMB shares.
# This tool is dedicated for advanced Unix users and programmers.
# 
# For more info on this product read Zaval File Search User's Guide
# (It comes within this package).
# The latest product version is always available from the product's homepage:
# http://www.zaval.org/products/file-search/
# and from the SourceForge:
# http://sourceforge.net/projects/zaval0001/
#
# Contacts:
#   Author : v.krapivin@zaval.org
#   Support : support@zaval.org
#   Change Requests : change-request@zaval.org
# 
# Copyright (C) 2001-2002 Zaval Creative Engineering Group (http://www.zaval.org)
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# (version 2) as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

# Generic routine to calculate directory size


use integer;

$confdir="/etc/zfsearch";

sub calcsize {
    my $dirname=$_[0];
    my $size=$_[1];

    $i=3;
    while (1) {
        $i=rindex($dirname,"/");
        $dirsize{$dirname} += $size;
        $dirname=substr($dirname,0,$i);
        if (index("$dirname","://")<0) {last}
    }
}

# Generic routine to provide parallelized scan
# It makes separate sequences of network to perform
# these sequences in parallel.

# The generic algorithm is the each call of this script contains
# the portion number and total amount of portions. The all actions
# below are applied only for proper host list projectin created by.
# The real parallelized startup is provided by rounding shell script.

sub intlvl {
    if ($ARGV[1]>0) {
        my $lp=$ARGV[1];
        my $cl=$ARGV[0];
        if ($cl>=$lp) {return}
        @hostlist=();
        @ARGV=();
        $i=0;
        while (<>) { chomp;
            if ($i == $lp) {$i=0}
            if ($i == $cl) {push(@hostlist,$_)}
            $i++;
        }
    } else {
        @ARGV=();
        while (<>) { chomp;
            push(@hostlist,$_);
        }
    }
}

sub norescan() {
    ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($fn);
    if ($currtime-$mtime < $seconds_rescan) {
        print "\tnorescan $host\n";
        return 1;
    } else {return 0}
}

sub testexcludeshare($) {
    local $s=shift;
    return ( grep( $s =~ /$_/i, @excludeshares ) ? 1 : 0 );
}
sub testexcludefile($) {
    local $s=shift;
    return ( grep( $s =~ /$_/i, @excludefiles ) ? 1 : 0 );
}

sub scansmb() {
    # Pass one: get the shares related to host currently scanning.
    # The all shares (including hidden) will be tried to scan. But the
    # system shares with '$' mark will be skipped in all cases.

    $host=$1;
    $fn="$datadir/list.smb.$host";
    if (norescan()==1) {return;}

    print "\tListing smb://$host\n";

    @shares=();
    open(SL,"smbclient \"//".$host."/ipc\$\" -A /etc/zfsearch/smbclient.conf -L \"".$host."\" |") or die "Can't run smbclient";
    $allow=0;
    while (<SL>) { chomp;
        if (/^\s+(-+)\s+(-+)\s+(-+)\s*$/) {next;}
        if (/^\s*$/) {$allow=0;}
        if ($allow && /^\s+(\S+)\s+(\S+)\s*(\S*)\s*/) {
            if ($2 eq "Disk") {push(@shares,$1);}
        }
        if (/^\s+Sharename\s+Type\s+Comment\s*$/) {$allow=1;}
    }
    close(SL);
    $n=@shares;

    # Scanning all files related to current share. Of course, the share
    # can be scanned only if credentials enough to.

    if ($n>0) {
        open(O,"> $fn") or die "Can't create output file";
    }
    foreach $s (@shares) {
        if (testexcludeshare("smb://$host/$s")!=0) {next}
        print "\tScanning smb://$host/$s\n";
        # Every share have accumulated in its own file to avoid
        # file oplocks requirements during scanning process. The result
        # file will not be produced until all shares have scanned.

        # The result file has the following structure:
        # {File=>F, Dir=>D}:{date:optional}:size:full path

        open(L,"perl -e 'print \"recurse\nls\nexit\n\";' | smbclient \"//$host/$s\" -A /etc/zfsearch/smbclient.conf |") or die "Can't run smbclient";
        $allow=0;
        $dir="";
        %dirsize=();
        while (<L>) { chomp;
            if (/^directory recursion is now/) {next}
            if (/^smb/) {$allow=1}
            if ($allow==0) {next}
            if (/^\s*$/ || /^smb:/) {next}
            if (/^\s+(.+?)\s+(D?A?R?H?)\s+(\d+)\s+(\w+)\s+(\w+)\s+(\d+)\s+([\d|:]+)\s+(\d+)\s*$/) {
                if ($1 eq ".") {next}
                if ($1 eq "..") {next}
                $filename=$1;
                $attrs=$2;
                $size=$3;
                $dow=$4;
                $mon=$5;
                $dom=$6;
                $time=$7;
                $year=$8;

                if ($filename eq "." || $filename eq "..") {next}
                if ($attrs =~ /D/) {next}
                $monn=$monthes{$mon};
                $time =~ s/:/\./g;
                if (testexcludefile("smb://$host/$s$dir/$filename")!=0) {next}
                print O "F:$year-$monn-$dom-$time:$size:smb://$host/$s$dir/$filename\n";
                calcsize("smb://$host/$s$dir",$size);
            } else {
                s/\\/\//g;$dir=$_;
            }
        }
        foreach $d (sort keys %dirsize) {
            if (testexcludefile($d)!=0) {next}
            print O "D::$dirsize{$d}:$d\n";
        }
        close(L);
    }
    close(O);
}

use Net::FTP;

sub scanftp() {
    $host=$1;
    $fn="$datadir/list.ftp.$host";
    if (norescan()==1) {return;}

    # Scanning all files related to current ftp.
        if (testexcludeshare("ftp://$host")!=0) {next}
    
        print "\tScanning ftp://$host\n";

        # Every share have accumulated in its own file to avoid
        # file oplocks requirements during scanning process. The result
        # file will not be produced until all shares have scanned.

        # The result file has the following structure:
        # {File=>F, Dir=>D}:{date:optional}:size:full path

        $ftp = Net::FTP->new($host);
        if (defined $ftp) {
            $ftp->login($ENV{"FTP_USER"},$ENV{"FTP_PASSWORD"});
            @list=$ftp->ls("-lR");
            $dir="";
            %dirsize=();
            open(O,"> $fn") or die "Can't create output file $datadir/list.ftp.$host";
            foreach (@list) { chomp;
                if (/^\.*(.+):$/) { #dir
                    $dir=$1;
                } elsif (/^(.+)\s+\d+\s+.+\s+.+\s+(\d+)\s+(\w+)\s+(\d{1,2})\s+([\d|:]+)\s+(.+)/) {
                    $attrs=$1;
                    $size=$2;
                    $mon=$3;
                    $dom=$4;
                    $time=$5;
                    $year=$5;
                    $filename=$6;
                    if ($filename=~/^(.+) ->/) {
                        $filename=$1;
                    }

                    if ($attrs =~ /^d/) {next}
                    $monn=$monthes{$mon};
                    if ($year =~ /:/) {
                        $year=$currentyear;
                    } else {
                        $time="12:00";
                    }
                    $time =~ s/:/\./g;
                    if (testexcludefile("ftp://$host$dir/$filename")!=0) {next}
                    print O "F:$year-$monn-$dom-$time:$size:ftp://$host$dir/$filename\n";
                    calcsize("ftp://$host$dir",$size);
                }
            }
            $ftp->quit;
            foreach $d (sort keys %dirsize) {
                if (testexcludefile($d)!=0) {next}
                print O "D::$dirsize{$d}:$d\n";
            }
            close(O);
        }
}

# Generic date manipulations: month recoding (locale C)

%monthes = ("Jan"=>1, "Feb"=>2, "Mar"=>3, "Apr"=>4, "May"=>5, "Jun"=>6, "Jul"=>7 ,"Aug"=>8, "Sep"=>9, "Oct"=>10, "Nov"=>11, "Dec"=>12);

intlvl();

# Generic scan based upon smbclient calls.
# The all calls are user credentioals sensitive (see config and
# documentation fotr details.

$currtime=time;

($sec,$min,$hour,$mday,$mon,$currentyear,$wday,$yday,$isdst) = localtime($currtime);

$currentyear+=1900;

$datadir=$ENV{"DATADIR"}."/lists";
$seconds_remove=$ENV{"TIME_ALLOW"} * 60*60;
$seconds_rescan=$ENV{"TIME_RESCAN"} * 60*60;

open(E,"$confdir/exclude.shares");
while (<E>) { chomp;
    if (/^\s*#/) {next}
    if (/^\s*$/) {next}
    push(@excludeshares,$_);
}
close(E);
open(E,"$confdir/exclude.files");
while (<E>) { chomp;
    if (/^\s*#/) {next}
    if (/^\s*$/) {next}
    push(@excludefiles,$_);
}
close(E);

foreach $host (@hostlist) {
    if ($host =~ /^smb:\/\/(.+)/) {scansmb();}
    if ($host =~ /^ftp:\/\/(.+)/) {scanftp();}
}
